import 'dart:ffi';

import 'package:ffi/ffi.dart';
import 'package:win32/win32.dart';

import 'base.dart';
import 'com/IMetaDataImport2.dart';
import 'mixins/customattributes_mixin.dart';
import 'win32.dart';

class Event extends TokenObject with CustomAttributesMixin {
  final String eventName;
  final int classToken;
  final int attributes;
  final int eventType;
  final int addOnToken;
  final int removeOnToken;
  final int fireToken;
  final List<int> otherMethodTokens;

  Event(
      IMetaDataImport2 reader,
      int token,
      this.eventName,
      this.classToken,
      this.attributes,
      this.eventType,
      this.addOnToken,
      this.removeOnToken,
      this.fireToken,
      this.otherMethodTokens)
      : super(reader, token);

  factory Event.fromToken(IMetaDataImport2 reader, int token) {
    final ptkClass = calloc<mdTypeDef>();
    final szEvent = stralloc(MAX_STRING_SIZE);
    final pchEvent = calloc<ULONG>();
    final pdwEventFlags = calloc<DWORD>();
    final ptkEventType = calloc<mdToken>();
    final ptkAddOn = calloc<mdMethodDef>();
    final ptkRemoveOn = calloc<mdMethodDef>();
    final tkkFire = calloc<mdMethodDef>();
    final rgOtherMethod = calloc<mdMethodDef>(16);
    final pcOtherMethod = calloc<ULONG>();

    try {
      final hr = reader.GetEventProps(
          token,
          ptkClass,
          szEvent,
          MAX_STRING_SIZE,
          pchEvent,
          pdwEventFlags,
          ptkEventType,
          ptkAddOn,
          ptkRemoveOn,
          tkkFire,
          rgOtherMethod,
          16,
          pcOtherMethod);

      if (SUCCEEDED(hr)) {
        return Event(
            reader,
            token,
            szEvent.toDartString(),
            ptkClass.value,
            pdwEventFlags.value,
            ptkEventType.value,
            ptkAddOn.value,
            ptkRemoveOn.value,
            tkkFire.value,
            rgOtherMethod.asTypedList(pcOtherMethod.value));
      } else {
        throw WindowsException(hr);
      }
    } finally {
      free(ptkClass);
      free(szEvent);
      free(pchEvent);
      free(pdwEventFlags);
      free(ptkEventType);
      free(ptkAddOn);
      free(ptkRemoveOn);
      free(tkkFire);
      free(rgOtherMethod);
      free(pcOtherMethod);
    }
  }
}
